@@ -17,28 +17,24 @@
 # imports
 ########################################################################
 
-import base64
 from   calendar import timegm
 import ConfigParser
-import cPickle
-import cStringIO
-import dircache
-import gzip
 import imp
 import lock
-import operator
 import os
 import os.path
 import qm
-import quopri
 import re
-import socket
 import string
 import sys
 import tempfile
 import time
 import traceback
 import types
+import getpass
+import StringIO
+import htmllib
+import formatter
 if sys.platform != "win32":
     import fcntl
     
@@ -53,13 +49,6 @@ program_name = None
 # exceptions
 ########################################################################
 
-class Empty:
-    """An empty class."""
-
-    pass
-
-
-
 class QMException(Exception):
     """An exception generated directly by QM.
 
@@ -76,26 +65,6 @@ class QMException(Exception):
 
 
 
-class MutexError(QMException):
-    """A problem occurred with a mutex."""
-
-    pass
-
-
-
-class MutexLockError(QMException):
-    """A lock was not obtained on the mutex."""
-
-    pass
-
-
-
-class ConfigurationError(QMException):
-
-    pass
-
-
-
 class UserError(QMException):
 
     pass
@@ -269,31 +238,12 @@ def format_traceback(exc_info):
     return string.join(traceback.format_tb(exc_info[2]), "\n")
 
 
-def format_byte_count(bytes):
-    """Return the traditional representation of 'bytes' bytes."""
-
-    kb = 1024.0
-    mb = kb * 1024
-    gb = mb * 1024
-    tb = gb * 1024
-    
-    for name, order in [
-        ("TB", tb),
-        ("GB", gb),
-        ("MB", mb),
-        ("KB", kb),
-        ]:
-        if bytes >= order:
-            return "%.1f %s" % (bytes / order, name)
-
-    return "%d bytes" % bytes
-
-
 def convert_from_dos_text(text):
     """Replace CRLF with LF in 'text'."""
 
     return string.replace(text, "\r\n", "\n")
 
+
 __load_module_lock = lock.RLock()
 """A lock used by load_module."""
 
@@ -447,66 +397,6 @@ def split_path_fully(path):
         return split_path_fully(dir) + [ entry ]
 
 
-def encode_data_as_text(data, mime_type="application/octet-stream"):
-    """Encode data as text.
-
-    'data' -- The data to encode.
-
-    'mime_type' -- The MIME type of the data.
-
-    returns -- A pair.  The first element designates the encoding scheme
-    used.  The second is a string containing the encoded data."""
-
-    base_type = string.split(mime_type, "/", 1)[0]
-
-    # For the text base MIME type, use a quoted-printable encoding.
-    # This makes the encoded data more human-friendly.
-    if base_type == "text":
-        encoding = "quoted-printable"
-        input_file = cStringIO.StringIO(data)
-        output_file = cStringIO.StringIO()
-        quopri.encode(input_file, output_file, 0)
-        data = output_file.getvalue()
-
-    # For everything else, gzip the data and then base64-encode it.
-    else:
-        encoding = "gzipped base64"
-        data = gzip.zlib.compress(data)
-        data = base64.encodestring(data)
-
-    return (encoding, data)
-
-
-def decode_data_from_text(data, encoding):
-    """Decode data that was encoded as text.
-
-    'data' -- The encoded data.
-
-    'encoding' -- The encoding scheme used to encode this data.
-
-    returns -- A string containing the decoded data."""
-
-    if encoding == "none":
-        return data
-
-    elif encoding == "quoted-printable":
-        # Decode quoted-printable text.
-        input_file = cStringIO.StringIO(data)
-        output_file = cStringIO.StringIO()
-        quopri.decode(input_file, output_file)
-        return output_file.getvalue()
-        
-    elif encoding == "gzipped base64":
-        # First base64-decode the data.
-        data = base64.decodestring(data)
-        # Now uncompress it.
-        return gzip.zlib.decompress(data)
-
-    else:
-        # Unknown encoding type.
-        raise ValueError, "unknown encoding %s" % encoding
-
-
 def open_temporary_file_fd(suffix = ""):
     """Create and open a temporary file.
 
@@ -598,18 +488,6 @@ def copy(object):
         return object
 
 
-def indent_lines(text, indent):
-    """Indent each line of 'text' by 'indent' spaces."""
-
-    indentation = ' ' * indent
-    # Break into lines.
-    lines = string.split(text, "\n")
-    # Indent each.
-    lines = map(lambda line, ind=indentation: ind + line, lines)
-    # Rejoin.
-    return string.join(lines, "\n")
-
-
 def wrap_lines(text, columns=72, break_delimiter="\\", indent=""):
     """Wrap lines in 'text' to 'columns' columns.
 
@@ -681,7 +559,7 @@ def format_time(time_secs, local_time_zone=1):
            "%(hour)02d:%(minute)02d %(time_zone)s" % locals()
 
 
-def format_time_iso(time_secs):
+def format_time_iso(time_secs=None):
     """Generate a ISO8601-compliant formatted date and time.
 
     The output is in the format "YYYY-MM-DDThh:mm:ss+TZ", where TZ is
@@ -690,13 +568,30 @@ def format_time_iso(time_secs):
     sorting behaviour.
 
     'time_secs' -- The time to be formatted, as returned by
-    e.g. 'time.time()'.
+    e.g. 'time.time()'.  If 'None' (the default), uses the current
+    time.
 
     returns -- The formatted time as a string."""
 
+    if time_secs is None:
+        time_secs = time.time()
     return time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(time_secs))
 
 
+def parse_time_iso(time_string):
+    """Parse a ISO8601-compliant formatted date and time.
+
+    See also 'format_time_iso'.
+
+    'time_string' -- The string to be parsed, as returned by
+    e.g. 'format_time_iso'.
+
+    returns -- The time as a float, like that returned by
+    'time.time'."""
+
+    return time.mktime(time.strptime(time_string, "%Y-%m-%dT%H:%M:%SZ"))
+
+
 def make_unique_tag():
     """Return a unique tag string."""
 
@@ -884,6 +779,74 @@ def read_assignments(file):
 
     return assignments
 
+
+def get_username():
+    """Returns the current username as a string.
+
+    This is our best guess as to the username of the user who is
+    actually logged in, as opposed to the effective user id used for
+    running tests.
+
+    If the username cannot be found, raises a 'QMException'."""
+
+    # First try using the 'getpass' module.
+    try:
+        return getpass.getuser()
+    except:
+        pass
+
+    # 'getpass' doesn't necessarily work on Windows, so if that fails,
+    # try the win32 function.
+    try:
+        import win32api
+    except ImportError:
+        pass
+    else:
+        try:
+            return win32api.GetUserName()
+        except:
+            raise PythonException("Error accessing win32 user database",
+                                  *sys.exc_info()[:2])
+
+    # And if none of that worked, give up.
+    raise QMException, "Cannot determine user name."
+
+
+def get_userid():
+    """Returns the current user id as an integer.
+
+    This is the real user id, not the effective user id, to better track
+    who is actually running the tests.
+
+    If the user id cannot be found or is not defined, raises a
+    'QMException'."""
+
+    try:
+        uid = os.getuid()
+    except AttributeError:
+        raise QMException, "User ids not supported on this system."
+    return uid
+    
+
+def html_to_text(html, width=72):
+    """Renders HTML to text in a simple way.
+
+    'html' -- A string containing the HTML code to be rendered.
+
+    'width' -- Column at which to word-wrap.  Default 72.
+
+    returns -- A string containing a plain text rendering of the
+    HTML."""
+
+    s = StringIO.StringIO()
+    w = formatter.DumbWriter(s, width)
+    f = formatter.AbstractFormatter(w)
+    p = htmllib.HTMLParser(f)
+    p.feed(html)
+    p.close()
+    return s.getvalue()
+
+
 ########################################################################
 # variables
 ########################################################################
@@ -900,12 +863,6 @@ rc = RcConfiguration()
 # The next number to be used when handing out unqiue tag strings.
 _unique_tag = 0
 
-# The string types available in this implementation of Python.
-try:
-    string_types = (types.StringType, types.UnicodeType)
-except AttributeError:
-    string_types = (types.StringType,)
-
 ########################################################################
 # Local Variables:
 # mode: python
