@@ -7,7 +7,7 @@
 # Contents:
 #   QMTest database class.
 #
-# Copyright (c) 2001, 2002 by CodeSourcery, LLC.  All rights reserved. 
+# Copyright (c) 2001, 2002, 2003 by CodeSourcery, LLC.  All rights reserved. 
 #
 # For license terms see the file COPYING.
 #
@@ -20,10 +20,16 @@
 import os.path
 import qm
 from   qm.common import *
-import qm.test.base
+import qm.extension
+import qm.fields
+from   qm.label import *
+from   qm.test.base import *
+from   qm.test.directory_suite import DirectorySuite
+from   qm.test.runnable import Runnable
+from   qm.test.test import Test
 
 ########################################################################
-# classes
+# Classes
 ########################################################################
 
 class ItemDescriptor:
@@ -56,13 +62,11 @@ class ItemDescriptor:
         'arguments' -- A dictionary mapping argument names to argument
         values.  These arguments will be provided to the extension class
         when the entity is constructed."""
-        
-        qm.test.base.validate_id(instance_id)
+
         self.__database = database
         self.__id = instance_id
         self.__class_name = class_name
         self.__arguments = arguments
-        self.__working_directory = None
         self.__item = None
         
 
@@ -91,7 +95,7 @@ class ItemDescriptor:
         returns -- The Python class object for the entity.  For example,
         for a 'TestDescriptor', this method returns the test class."""
 
-        raise qm.MethodShouldBeOverriddenError, "ItemDescriptor.GetClass"
+        raise NotImplementedError
     
 
     def GetClassArguments(self):
@@ -102,7 +106,7 @@ class ItemDescriptor:
 
         Derived classes should not override this method."""
 
-        return qm.test.base.get_class_arguments(self.GetClass())
+        return qm.extension.get_class_arguments(self.GetClass())
     
 
     def GetArguments(self):
@@ -129,56 +133,23 @@ class ItemDescriptor:
         returns -- An instance of the class returned by 'GetClass'."""
 
         if not self.__item:
-            self.__item = self._MakeItem()
-
+            extras = { Runnable.EXTRA_ID : self.GetId(),
+                       Runnable.EXTRA_DATABASE : self.GetDatabase() }
+            self.__item = self.GetClass()(self.GetArguments(), **extras)
+            
         return self.__item
     
+
+    def GetResources(self):
+        """Return the resources required by this item.
+
+        returns -- A sequence of resource names.  Each name indicates a
+        resource that must be available to this item."""
+
+        return self.GetArguments().get(Runnable.RESOURCE_FIELD_ID, [])
         
-    def SetWorkingDirectory(self, directory_path):
-        """Set the directory in which this item will execute.
-
-        'directory_path' -- A path.  When the entity is executed, it
-        will execute in this directory."""
-
-        self.__working_directory = directory_path
-
-
-    def GetWorkingDirectory(self):
-        """Return the directory in which this item will execute.
-
-        returns -- The directory in which this entity will execute, or
-        'None' if it will it execute in the current directory."""
-
-        return self.__working_directory
-
     # Helper functions.
 
-    def _MakeItem(self):
-        """Construct the entity itself.
-
-        returns -- An instance of the class returned by 'GetClass'."""
-
-        arguments = self.GetArguments().copy()
-
-        # Do some extra processing for test arguments.
-        for field in self.GetClassArguments():
-            name = field.GetName()
-
-            # Use a default value for each field for which an argument
-
-            # was not specified.
-            if not arguments.has_key(name):
-                arguments[name] = field.GetDefaultValue()
-
-        # Record the test or resource name.  Logically, this
-        # should be a paramter passed in to the test class
-        # constructor, but that would require changing all the
-        # existing test classes.
-        arguments["id"] = self.GetId()
-        
-        return apply(self.GetClass(), [], arguments)
-
-    
     def _Execute(self, context, result, method):
         """Execute the entity.
         
@@ -190,28 +161,13 @@ class ItemDescriptor:
         'method' -- The method name of the method on the entity that
         should be invoked to perform the execution."""
 
-        working_directory = self.GetWorkingDirectory()
-        old_working_directory = None
-        
-        try:
-            if working_directory:
-                # Remember the previous working directory so we can
-                # restore it.
-                old_working_directory = os.getcwd()
-                # Change to the working directory appropriate for this
-                # test.
-                os.chdir(working_directory)
-            # Get the item.
-            item = self.GetItem()
-            # Execute the indicated method.
-            if context:
-                eval("item.%s(context, result)" % method)
-            else:
-                eval("item.%s(result)" % method)
-        finally:
-            # Restore the working directory.
-            if old_working_directory:
-                os.chdir(old_working_directory)
+        # Get the item.
+        item = self.GetItem()
+        # Execute the indicated method.
+        if context is not None:
+            eval("item.%s(context, result)" % method)
+        else:
+            eval("item.%s(result)" % method)
 
 
 
@@ -222,10 +178,7 @@ class TestDescriptor(ItemDescriptor):
                  database,
                  test_id,
                  test_class_name,
-                 arguments,
-                 prerequisites={},
-                 categories=[],
-                 resources=[]):
+                 arguments):
         """Create a new test instance.
 
         'database' -- The 'Database' containing this test.
@@ -235,24 +188,17 @@ class TestDescriptor(ItemDescriptor):
         'test_class_name' -- The name of the test class of which this is
         an instance.
 
-        'arguments' -- This test's arguments to the test class.
-
-        'prerequisites' -- A mapping from prerequisite test ID to
-        required outcomes.
-
-        'categories' -- A sequence of names of categories to which this
-        test belongs.
-
-        'resources' -- A sequence of IDs of resources to run before and
-        after the test is run."""
+        'arguments' -- This test's arguments to the test class."""
 
         # Initialize the base class.
         ItemDescriptor.__init__(self, database,
                                 test_id, test_class_name, arguments)
-        self.__prerequisites = prerequisites
-        self.__categories = categories
-        self.__resources = resources
-        
+
+        self.__prerequisites = {}
+        for p, o in \
+            self.GetArguments().get(Test.PREREQUISITES_FIELD_ID, []):
+            self.__prerequisites[p] = o
+            
         # Don't instantiate the test yet.
         self.__test = None
 
@@ -263,9 +209,8 @@ class TestDescriptor(ItemDescriptor):
         returns -- The Python class object for the entity.  For example,
         for a 'TestDescriptor', this method returns the test class."""
 
-        return qm.test.base.get_extension_class(self.GetClassName(),
-                                                'test',
-                                                self.GetDatabase())
+        return get_extension_class(self.GetClassName(), 'test',
+                                   self.GetDatabase())
     
     
     def GetTest(self):
@@ -274,27 +219,12 @@ class TestDescriptor(ItemDescriptor):
         return self.GetItem()
 
 
-    def GetCategories(self):
-        """Return the names of categories to which the test belongs."""
-
-        return self.__categories
-    
-
     def GetPrerequisites(self):
         """Return a map from prerequisite test IDs to required outcomes."""
 
         return self.__prerequisites
 
 
-    def GetResources(self):
-        """Return the resources required by this test.
-
-        returns -- A sequence of resource names.  Each name indicates a
-        resource that must be available to this test."""
-
-        return self.__resources
-
-
     def GetTargetGroup(self):
         """Returns the pattern for the targets that can run this test.
 
@@ -339,8 +269,6 @@ class ResourceDescriptor(ItemDescriptor):
         # Initialize the base class.
         ItemDescriptor.__init__(self, database, resource_id,
                                 resource_class_name, arguments)
-        # Don't instantiate the resource yet.
-        self.__resource = None
 
 
     def GetClass(self):
@@ -349,9 +277,8 @@ class ResourceDescriptor(ItemDescriptor):
         returns -- The Python class object for the entity.  For example,
         for a 'TestDescriptor', this method returns the test class."""
 
-        return qm.test.base.get_extension_class(self.GetClassName(),
-                                                'resource',
-                                                self.GetDatabase())
+        return get_extension_class(self.GetClassName(), 'resource',
+                                   self.GetDatabase())
 
 
     def GetResource(self):
@@ -391,7 +318,25 @@ class DatabaseError(QMException):
     for the user."""
             
 
-class NoSuchTestError(DatabaseError):
+class NoSuchItemError(DatabaseError):
+    """An exception indicating that a particular item could not be found."""
+
+    def __init__(self, kind, item_id):
+
+        self.kind = kind
+        self.item_id = item_id
+
+
+    def __str__(self):
+        """Return a string describing this exception."""
+
+        return qm.message("no such item",
+                          kind = self.kind,
+                          item_id = self.item_id)
+
+
+
+class NoSuchTestError(NoSuchItemError):
     """The specified test does not exist."""
 
     def __init__(self, test_id):
@@ -399,17 +344,11 @@ class NoSuchTestError(DatabaseError):
 
         'test_id' -- The name of the test that does not exist."""
 
-        self.test_id = test_id
-
-
-    def __str__(self):
-        """Return a string describing this exception."""
-
-        return qm.error("no such test", test_id=test_id)
+        NoSuchItemError.__init__(self, Database.TEST, test_id)
 
                         
 
-class NoSuchSuiteError(DatabaseError):
+class NoSuchSuiteError(NoSuchItemError):
     """The specified suite does not exist."""
 
     def __init__(self, suite_id):
@@ -417,17 +356,11 @@ class NoSuchSuiteError(DatabaseError):
 
         'suite_id' -- The name of the suite that does not exist."""
 
-        self.suite_id = suite_id
-
-        
-    def __str__(self):
-        """Return a string describing this exception."""
-
-        return qm.error("no such suite", suite_id=self.suite_id)
+        NoSuchItemError.__init__(self, Database.SUITE, suite_id)
 
 
 
-class NoSuchResourceError(DatabaseError):
+class NoSuchResourceError(NoSuchItemError):
     """The specified resource does not exist."""
 
     def __init__(self, resource_id):
@@ -435,16 +368,11 @@ class NoSuchResourceError(DatabaseError):
 
         'resource_id' -- The name of the resource that does not exist."""
 
-        self.resource_id = resource_id
-
-    def __str__(self):
-        """Return a string describing this exception."""
-
-        return qm.error("no such resource", resource_id=self.resource_id)
+        NoSuchItemError.__init__(self, Database.RESOURCE, resource_id)
 
 
 
-class Database:
+class Database(qm.extension.Extension):
     """A 'Database' stores tests, testsuites, and resources.
 
     A 'Database' has two primary functions:
@@ -520,24 +448,141 @@ class Database:
     simultaneously.  Therefore, you must take appropriate steps to
     ensure thread-safe access to shared data."""
 
-    def __init__(self, path, **attributes):
+    arguments = [
+        qm.fields.TextField(
+            name = "label_class",
+            title = "Label Class",
+            description = """The name of the label class used by this database.
+
+            The label class is used to separate names of entities used
+            by the database into directories and basenames.""",
+            default_value = "python_label.PythonLabel"
+            ),
+        qm.fields.BooleanField(
+            name = "modifiable",
+            title = "Modifiable?",
+            description = """Whether or not the database can be modified.
+
+            If true, changes (such as the addition or removal of tests,
+            resources, or suites) can be made to the test database.
+            If false, tests can be viewed and run, but not modified.""",
+            default_value = "true")
+        ]
+
+    RESOURCE = "resource"
+    SUITE = "suite"
+    TEST = "test"
+    
+    ITEM_KINDS = [RESOURCE, SUITE, TEST]
+    """The kinds of items that can be stored in a 'Database'."""
+
+    _item_exceptions = {
+        RESOURCE : NoSuchResourceError,
+        SUITE : NoSuchSuiteError,
+        TEST : NoSuchTestError
+        }
+    """The exceptions to be raised when a particular item cannot be found.
+
+    This map is indexed by the 'ITEM_KINDS'; the value indicates the
+    exception class to be used when the indicated kind cannot be found."""
+    
+    kind = "database"
+    """The 'Extension' kind."""
+
+    def __init__(self, path, arguments):
         """Construct a 'Database'.
 
         'path' -- A string containing the absolute path to the directory
         containing the database.
 
-        'attributes' -- A dictionary mapping attribute names to values.
+        'arguments' -- A dictionary mapping attribute names to values.
         
         Derived classes must call this method from their own '__init__'
-        methods.  Evey derived class must have an '__init__' method that
+        methods.  Every derived class must have an '__init__' method that
         takes the path to the directory containing the database as its
         only argument.  The path provided to the derived class '__init__'
         function will always be an absolute path."""
 
+        qm.extension.Extension.__init__(self, arguments)
+        
         # The path given must be an absolute path.
         assert os.path.isabs(path)
         self.__path = path
+
+        # Translate the label class name into an actual Python class.
+        self.__label_class \
+            = get_extension_class(self.label_class, "label", self)
+                                          
+    # Methods that deal with labels.
+    
+    def IsValidLabel(self, label, is_component = 1):
+        """Return true if 'label' is valid.
+
+        'label' -- A string that is being considered as a label.
+
+        'is_component' -- True if the string being tested is just a
+        single component of a label path.
         
+        returns -- True if 'label' is a valid name for entities in this
+        database."""
+
+        return self.__label_class("").IsValid(label, is_component)
+
+
+    def JoinLabels(self, *labels):
+        """Join the 'labels' together.
+
+        'labels' -- A sequence of strings corresponding to label
+        components.
+
+        returns -- A string containing the complete label."""
+
+        if not labels:
+            return ""
+
+        return str(apply(self.__label_class(labels[0]).Join,
+                         labels[1:]))
+    
+
+    def SplitLabel(self, label):
+        """Split the label into a pair '(directory, basename)'.
+
+        returns -- A pair of strings '(directory, basename)'."""
+
+        return map(str, self.__label_class(label).Split())
+
+
+    def SplitLabelLeft(self, label):
+        """Split the label into a pair '(parent, subpath)'.
+        This is the same operation as SplitLabel, except the split
+        occurs at the leftmost separator, not the rightmost, and a
+        single-component label comes back in the parent slot.
+
+        returns -- A pair of strings '(parent, subpath)'."""
+
+        return map(str, self.__label_class(label).SplitLeft())
+
+
+    def GetLabelComponents(self, label):
+        """Return all of the component directories of 'label'.
+
+        'label' -- A string naming an entity in the database.
+
+        returns -- A list of strings.  The first string is the first
+        directory in 'label'; the last string is the basename."""
+
+        components = []
+        while label:
+            dirname, label = self.SplitLabelLeft(label)
+            if dirname:
+                components.append(dirname)
+            else:
+                components.append(label)
+                break
+
+        return components
+    
+                
     # Methods that deal with tests.
     
     def GetTest(self, test_id):
@@ -548,51 +593,9 @@ class Database:
         returns -- A 'TestDescriptor' corresponding to 'test_id'.
         
         raises -- 'NoSuchTestError' if there is no test in the database
-        named 'test_id'.
+        named 'test_id'."""
 
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.GetTest"
-
-
-    def WriteTest(self, test):
-        """Store 'test' in the database.
-
-        'test' -- A 'TestDescriptor' indicating the test that should
-        be stored.
-
-        'Attachment's associated with 'test' may be located in the
-        'AttachmentStore' associated with this database, or in some
-        other 'AttachmentStore'.  In the case that they are stored
-        elsewhere, they must be copied into the 'AttachmentStore'
-        associated with this database by use of the
-        'AttachmentStore.Store' method.  The caller, not this method,
-        is responsible for removing the original version of the
-        attachment, if necessary.
-         
-        The 'test' may be new, or it may be a new version of an existing
-        test.  If it is a new version of an existing test, the database
-        may wish to clear out any storage associated with the existing
-        test.  However, it is possible that 'Attachment's associated
-        with the existing test are still present in 'test', in which
-        case it would be a mistake to remove them.
-
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.WriteTest"
-
-
-    def RemoveTest(self, test_id):
-        """Remove the test named 'test_id' from the database.
-
-        'test_id' -- A label naming the test that should be removed.
-
-        raises -- 'NoSuchTestError' if there is no test in the database
-        named 'test_id'.
-
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.RemoveTest"
+        raise NoSuchTestError(test_id)
 
 
     def HasTest(self, test_id):
@@ -618,7 +621,7 @@ class Database:
             return 1
 
 
-    def GetTestIds(self, directory=".", scan_subdirs=1):
+    def GetTestIds(self, directory="", scan_subdirs=1):
         """Return all test IDs that begin with 'directory'.
 
         'directory' -- A label indicating the directory in which to
@@ -628,11 +631,9 @@ class Database:
         'directory' should be scanned.
         
         'returns' -- A list of all tests located within 'directory',
-        as absolute labels.
+        as absolute labels."""
 
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.GetTestIds"
+        return self.GetIds(self.TEST, directory, scan_subdirs)
 
     # Methods that deal with suites.
 
@@ -647,47 +648,16 @@ class Database:
         raises -- 'NoSuchSuiteError' if there is no test in the database
         named 'test_id'.
 
-        All databases must have an implicit suite called '.' that
+        All databases must have an implicit suite called '' that
         contains all tests in the database.  More generally, for each
         directory in the database, there must be a corresponding suite
         that contains all tests in that directory and its
-        subdirectories.
+        subdirectories."""
 
-        Derived classes must override this method."""
+        if suite_id == "":
+            return DirectorySuite(self, "")
 
-        raise qm.MethodShouldBeOverriddenError, "Database.GetSuite"
-
-
-    def WriteSuite(self, suite):
-        """Store 'suite' in the database.
-
-        'suite' -- An instance of 'Suite' (or a derived class of
-        'Suite') that should be stored.  The 'suite' will not be
-        implicit.
-
-        The 'suite' may be new, or it may be a new version of an
-        existing testsuite.  If 'suite' is a new version of an existing
-        suite, it may name fewer tests than the existing version.
-        However, this method should not remove any of the tests
-        themselves.
-
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.WriteSuite"
-
-
-    def RemoveSuite(self, suite_id):
-        """Remove the suite named 'suite_id' from the database.
-
-        'suite_id' -- A label naming the suite that should be removed.
-        The suite will not be implicit.
-          
-        raises -- 'NoSuchSuiteError' if there is no suite in the
-        database named 'test_id'.
-
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.RemoveSuite"
+        raise NoSuchSuiteError(suite_id)
 
 
     def HasSuite(self, suite_id):
@@ -703,7 +673,7 @@ class Database:
         between the time that 'HasSuite' is called and the time that
         'GetSuite' is called.
 
-        All databases must have an implicit suite called '.' that
+        All databases must have an implicit suite called "" that
         contains all tests in the database.  More generally, for each
         directory in the database, there must be a corresponding suite
         that contains all tests in that directory and its
@@ -719,7 +689,7 @@ class Database:
             return 1
 
 
-    def GetSuiteIds(self, directory=".", scan_subdirs=1):
+    def GetSuiteIds(self, directory="", scan_subdirs=1):
         """Return all suite IDs that begin with 'directory'.
 
         'directory' -- A label indicating the directory in which to
@@ -729,11 +699,9 @@ class Database:
         'directory' should be scanned.
 
         'returns' -- A list of all suites located within 'directory',
-        as absolute labels.
+        as absolute labels."""
 
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.GetSuiteIds"
+        return self.GetIds(self.SUITE, directory, scan_subdirs)
 
 
     # Methods that deal with resources.
@@ -746,39 +714,9 @@ class Database:
         returns -- A 'ResourceDescriptor' corresponding to 'resource_id'.
         
         raises -- 'NoSuchResourceError' if there is no resource in the
-        database named 'resource_id'.
+        database named 'resource_id'."""
 
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.GetResource"
-
-
-    def WriteResource(self, resource):
-        """Store 'resource' in the database.
-
-        'resource' -- A 'ResourceDescriptor' indicating the resource that
-        should be stored.
-
-        The 'resource' may be new, or it may be a new version of an
-        existing resource.
-
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.WriteResource"
-
-
-    def RemoveResource(self, resource_id):
-        """Remove the resource named 'resource_id' from the database.
-
-        'resource_id' -- A label naming the resource that should be
-        removed.
-
-        raises -- 'NoSuchResourceError' if there is no resource in the
-        database named 'resource_id'.
-
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.RemoveResource"
+        raise NoSuchResourceError(resource_id)
 
 
     def HasResource(self, resource_id):
@@ -805,7 +743,7 @@ class Database:
             return 1
 
 
-    def GetResourceIds(self, directory=".", scan_subdirs=1):
+    def GetResourceIds(self, directory="", scan_subdirs=1):
         """Return all resource IDs that begin with 'directory'.
 
         'directory' -- A label indicating the directory in which to
@@ -815,14 +753,50 @@ class Database:
         'directory' should be scanned.
 
         'returns' -- A list of all resources located within 'directory',
-        as absolute labels.
+        as absolute labels."""
 
-        Derived classes must override this method."""
-
-        raise qm.MethodShouldBeOverriddenError, "Database.GetResourceIds"
+        return self.GetIds(self.RESOURCE, directory, scan_subdirs)
 
     # Miscellaneous methods.
 
+    def GetIds(self, kind, directory = "", scan_subdirs = 1):
+        """Return all IDs of the indicated 'kind' that begin with 'directory'.
+
+        'kind' -- One of the 'ITEM_KINDS'.
+        
+        'directory' -- A label indicating the directory in which to
+        begin the search.
+
+        'scan_subdirs' -- True if (and only if) subdirectories of
+        'directory' should be scanned.
+
+        returns -- A list of all items of the indicated 'kind' located
+        within 'directory', as absolute labels.
+
+        Derived classes may override this method."""
+
+        return []
+
+
+    def GetItem(self, kind, item_id):
+        """Return the item of the indicated 'kind' with indicated 'item_id'.
+
+        'kind' -- One of the 'ITEM_KINDS'.
+
+        'item_id' -- The name of the item.
+
+        returns -- If 'kind' is 'Database.TEST' or 'Database.RESOURCE',
+        returns a test descriptor or resource descriptor, respectively.
+        If 'kind' is 'Database.SUITE', returns a 'Suite'.
+
+        Derived classes may override this method."""
+
+        return { Database.TEST : self.GetTest,
+                 Database.RESOURCE : self.GetResource,
+                 Database.SUITE : self.GetSuite } [kind] (item_id)
+
+    
+        
     def GetSubdirectories(self, directory):
         """Return the immediate subdirectories of 'directory'.
 
@@ -833,9 +807,9 @@ class Database:
         and "a.c" are directories in the database, this method will
         return "b" and "c" given "a" as 'directory'.
 
-        Derived classes must override this method."""
+        Derived classes may override this method."""
 
-        raise qm.MethodShouldBeOverriddenError, "Database.GetSubdirectories"
+        return []
         
         
     def GetPath(self):
@@ -849,15 +823,26 @@ class Database:
         return self.__path
     
 
+    def GetConfigurationDirectory(self):
+        """Return the directory containing configuration information.
+
+        returns -- The directory containing configuration information
+        for the database.
+
+        Derived classes must not override this method."""
+
+        return get_configuration_directory(self.GetPath())
+    
+
     def GetAttachmentStore(self):
         """Returns the 'AttachmentStore' associated with the database.
 
         returns -- The 'AttachmentStore' containing the attachments
         associated with tests and resources in this database.
 
-        Derived classes must override this method."""
+        Derived classes may override this method."""
 
-        raise qm.MethodShouldBeOverriddenError, "Database.GetAttachmentStore"
+        return None
 
 
     def GetClassPaths(self):
@@ -878,13 +863,7 @@ class Database:
         returned by the default implementation (but probably should
         be)."""
 
-        # Specify the configuration subdirectory.
-        config_dir = qm.test.base.get_db_configuration_directory(
-            self.GetPath())
-        # It should exist.
-        assert os.path.isdir(config_dir)
-        # That's where test and resources classes go.
-        return [config_dir]
+        return []
 
 
     def GetTestClassNames(self):
@@ -898,7 +877,7 @@ class Database:
         implementation allows all available test classes, but the
         derived class may allow only a subset."""
 
-        return qm.test.base.get_extension_class_names('test', self)
+        return get_extension_class_names('test', self)
 
 
     def GetResourceClassNames(self):
@@ -912,7 +891,7 @@ class Database:
         implementation allows all available resource classes, but the
         derived class may allow only a subset."""
 
-        return qm.test.base.get_extension_class_names('resource', self)
+        return get_extension_class_names('resource', self)
 
 
     def ExpandIds(self, ids):
@@ -934,11 +913,6 @@ class Database:
         # checks efficient.
         test_ids = {}
         suite_ids = {}
-        # These function add to the maps.
-        def add_test_id(test_id, test_ids=test_ids):
-            test_ids[test_id] = None
-        def add_suite_id(suite_id, suite_ids=suite_ids):
-            suite_ids[suite_id] = None
 
         for id in ids:
             # Skip this ID if we've already seen it.
@@ -946,19 +920,21 @@ class Database:
                 continue
             # Is this a suite ID?
             if self.HasSuite(id):
-                add_suite_id(id)
+                suite_ids[id] = None
                 # Yes.  Load the suite.
                 suite = self.GetSuite(id)
                 # Determine all the tests and suites contained directly and
                 # indirectly in this suite.
-                suite_test_ids, sub_suite_ids = suite.GetAllTestAndSuiteIds()     
+                suite_test_ids, sub_suite_ids = suite.GetAllTestAndSuiteIds()
                 # Add them.
-                map(add_test_id, suite_test_ids)
-                map(add_suite_id, sub_suite_ids)
+                for test_id in suite_test_ids:
+                    test_ids[test_id] = None
+                for suite_id in sub_suite_ids:
+                    suite_ids[suite_id] = None
             # Or is this a test ID?
             elif self.HasTest(id):
                 # Yes.  Add it.
-                add_test_id(id)
+                test_ids[id] = None
             else:
                 # It doesn't look like a test or suite ID.
                 raise ValueError, id
@@ -966,6 +942,123 @@ class Database:
         # Convert the maps to sequences.
         return test_ids.keys(), suite_ids.keys()
 
+
+    def RemoveExtension(self, id, kind):
+        """Remove the extension 'id' from the database.
+
+        'id' -- A label for the 'Extension' instance stored in the
+        database.
+
+        'kind' -- The kind of 'Extension' stored with the given 'id'.
+        Some databases store different kinds of 'Extension' in different
+        namespaces so that it is possible for there to be more than one
+        'Extension' with the same 'id' in a single database."""
+
+        raise NotImplementedError
+        
+        
+    def WriteExtension(self, id, extension):
+        """Store 'extension' in the database, using the name 'id'.
+
+        'id' -- A label for the 'extension'.
+        
+        'extension' -- An instance of 'Extension'.
+
+        The 'extension' is stored in the database.  If there is a
+        previous item in the database with the same id', it is removed
+        and replaced with 'extension'.  Some databases may not be able
+        to store all 'Extension' instances; those database must throw an
+        exception when an attempt is made to store such an
+        'extension'."""
+
+        raise NotImplementedError
+        
+        
+    def IsModifiable(self):
+        """Returns true iff this database is modifiable.
+
+        returns -- True iff this database is modifiable.  If the
+        database is modifiable, it supports operatings like 'Write'
+        that make changes to the structure of the databaes itself.
+        Otherwise, the contents of the database may be viewed, but not
+        modified."""
+
+        return self.modifiable == "true"
+        
+########################################################################
+# Functions
+########################################################################
+
+def get_configuration_directory(path):
+    """Return the configuration directory for the 'Database' rooted at 'path'.
+
+    'path' -- The path to the test database.
+
+    returns -- The path to the configuration directory."""
+
+    return os.path.join(path, "QMTest")
+
+
+def get_configuration_file(path):
+    """Return the configuration file for the 'Database' rooted at 'path'.
+
+    'path' -- The path to the test database.
+
+    returns -- The path to the configuration file."""
+
+    return os.path.join(get_configuration_directory(path),
+                        "configuration")
+
+
+def is_database(db_path):
+    """Returns true if 'db_path' looks like a test database."""
+
+    # A test database is a directory.
+    if not os.path.isdir(db_path):
+        return 0
+    # A test database contains a configuration file.
+    if not os.path.isfile(get_configuration_file(db_path)):
+        return 0
+    # It probably is OK.
+    return 1
+
+
+def load_database(db_path):
+    """Load the database from 'db_path'.
+
+    'db_path' -- The path to the directory containing the database.
+    
+    returns -- The new 'Database'."""
+
+    # Make sure it is a directory.
+    if not is_database(db_path):
+        raise QMException, \
+              qm.error("not test database", path=db_path)
+
+    # Load the file.
+    config_path = get_configuration_file(db_path)
+    document = qm.xmlutil.load_xml_file(config_path)
+
+    # Parse it.
+    database_class, arguments \
+        = (qm.extension.parse_dom_element
+           (document.documentElement,
+            lambda n: qm.test.base.get_extension_class(n, "database",
+                                                       None, db_path)))
+    # For backwards compatibility with QM 1.1.x, we look for "attribute"
+    # elements.
+    for node in document.documentElement.getElementsByTagName("attribute"):
+        name = node.getAttribute("name")
+        # These elements were only allowed to contain strings as
+        # values.
+        value = qm.xmlutil.get_dom_text(node)
+        # Python does not allow keyword arguments to have Unicode
+        # values, so we convert the name to an ordinary string.
+        arguments[str(name)] = value
+        
+    return database_class(db_path, arguments)
+    
+    
 ########################################################################
 # Local Variables:
 # mode: python
