@@ -21,6 +21,7 @@ import qm
 from   qm.test.context import ContextException
 import sys
 import types
+import cgi
 
 ########################################################################
 # Classes
@@ -106,6 +107,8 @@ class Result:
     RESOURCE = "qmtest.resource"
     TARGET = "qmtest.target"
     TRACEBACK = "qmtest.traceback"
+    START_TIME = "qmtest.start_time"
+    END_TIME = "qmtest.end_time"
     
     # Other class variables.
 
@@ -259,6 +262,20 @@ class Result:
             return ""
     
         
+    def Quote(self, string):
+        """Return a version of string suitable for an annotation value.
+
+        Performs appropriate quoting for a string that should be taken
+        verbatim; this includes HTML entity escaping, and addition of
+        <pre> tags.
+
+        'string' -- The verbatim string to be quoted.
+
+        returns -- The quoted string."""
+
+        return "<pre>%s</pre>" % cgi.escape(string)
+
+
     def NoteException(self,
                       exc_info = None,
                       cause = None,
@@ -299,7 +316,7 @@ class Result:
         self[Result.CAUSE] = cause
         self[Result.EXCEPTION] = "%s: %s" % exc_info[:2]
         self[Result.TRACEBACK] \
-            = "<pre>" + qm.format_traceback(exc_info) + "</pre>"
+            = self.Quote(qm.format_traceback(exc_info))
 
         
     def MakeDomNode(self, document):
@@ -338,28 +355,28 @@ class Result:
     # annotations.
     
     def __getitem__(self, key):
-        assert type(key) in qm.common.string_types
+        assert type(key) in types.StringTypes
         return self.__annotations[key]
 
 
     def __setitem__(self, key, value):
-        assert type(key) in qm.common.string_types
-        assert type(value) in qm.common.string_types
+        assert type(key) in types.StringTypes
+        assert type(value) in types.StringTypes
         self.__annotations[key] = value
 
 
     def __delitem__(self, key):
-        assert type(key) in qm.common.string_types
+        assert type(key) in types.StringTypes
         del self.__annotations[key]
 
 
     def get(self, key, default=None):
-        assert type(key) in qm.common.string_types
+        assert type(key) in types.StringTypes
         return self.__annotations.get(key, default)
 
 
     def has_key(self, key):
-        assert type(key) in qm.common.string_types
+        assert type(key) in types.StringTypes
         return self.__annotations.has_key(key)
 
 
