@@ -18,8 +18,10 @@
 ########################################################################
 
 import qm
+import qm.common
 import qm.extension
 import qm.platform
+import qm.test.base
 from   qm.test.context import *
 from   qm.test.result import *
 from   qm.test.database import NoSuchResourceError
@@ -169,6 +171,7 @@ class Target(qm.extension.Extension):
         self.__engine = engine
         # There are no resources available on this target yet.
         self.__resources = {}
+        self.__order_of_resources = []
 
         
     def Stop(self):
@@ -181,12 +184,15 @@ class Target(qm.extension.Extension):
         Derived classes may override this method."""
         
         # Clean up any available resources.
-        for (name, rop) in self.__resources.items():
+        self.__order_of_resources.reverse()
+        for name in self.__order_of_resources:
+            rop = self.__resources[name]
             if rop and rop[1] == Result.PASS:
                 self._CleanUpResource(name, rop[0])
         del self.__response_queue
         del self.__engine
         del self.__resources
+        del self.__order_of_resources
 
 
     def RunTest(self, descriptor, context):
@@ -211,8 +217,14 @@ class Target(qm.extension.Extension):
             self.__SetUpResources(descriptor, context)
             # Make the ID of the test available.
             context[context.ID_CONTEXT_PROPERTY] = descriptor.GetId()
+            # Note the start time.
+            result[Result.START_TIME] = qm.common.format_time_iso()
             # Run the test.
-            descriptor.Run(context, result)
+            try:
+                descriptor.Run(context, result)
+            finally:
+                # Note the end time.
+                result[Result.END_TIME] = qm.common.format_time_iso()
         except KeyboardInterrupt:
             result.NoteException(cause = "Interrupted by user.")
             # We received a KeyboardInterrupt, indicating that the
@@ -313,6 +325,7 @@ class Target(qm.extension.Extension):
         del properties[Context.ID_CONTEXT_PROPERTY]
         rop = (resource, result.GetOutcome(), properties)
         self.__resources[result.GetId()] = rop
+        self.__order_of_resources.append(result.GetId())
         return rop
 
 
@@ -383,6 +396,9 @@ class Target(qm.extension.Extension):
             result.NoteException(cause="Resource is missing from the database.")
             self._RecordResult(result)
             return (None, result, None)
+        except qm.test.base.CouldNotLoadExtensionError, e:
+            result.NoteException(e.exc_info,
+                                 cause = "Could not load extension class")
         except KeyboardInterrupt:
             result.NoteException()
             # We received a KeyboardInterrupt, indicating that the
