@@ -7,7 +7,7 @@
 # Contents:
 #   Common code for implementing web user interfaces.
 #
-# Copyright (c) 2001, 2002 by CodeSourcery, LLC.  All rights reserved. 
+# Copyright (c) 2001, 2002, 2003 by CodeSourcery, LLC.  All rights reserved. 
 #
 # For license terms see the file COPYING.
 #
@@ -37,22 +37,15 @@ import socket
 import string
 import structured_text
 import sys
+import temporary_directory
 import time
 import traceback
 import types
 import urllib
 import user
 import whrandom
-import xmlrpclib
 
-# If the binary modules that are used in the DTML implementation are
-# linked against more recent versions of the C library than are found on
-# the present system, we will get an ImportError.  Catch the exception
-# and pass it on.
-try:
-    import DocumentTemplate
-except ImportError, message:
-    raise qm.common.QMException, message
+import qm.external.DocumentTemplate as DocumentTemplate
 
 ########################################################################
 # constants
@@ -65,22 +58,22 @@ session_id_field = "session"
 # exception classes
 ########################################################################
 
-class AddressInUseError(RuntimeError):
+class AddressInUseError(common.QMException):
     pass
 
 
 
-class PrivilegedPortError(RuntimeError):
+class PrivilegedPortError(common.QMException):
     pass
 
 
 
-class NoSessionError(RuntimeError):
+class NoSessionError(common.QMException):
     pass
 
 
 
-class InvalidSessionError(RuntimeError):
+class InvalidSessionError(common.QMException):
     pass
 
 
@@ -194,7 +187,7 @@ class DtmlPage:
         """Return the XML header for the document."""
 
         return \
-            '''<?xml version="1.0" encoding="UTF-8"?>
+            '''<?xml version="1.0" encoding="iso-8859-1"?>
                <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
                "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
                <html xmlns="http://www.w3.org/1999/xhtml">'''
@@ -238,12 +231,7 @@ class DtmlPage:
         result = """
         <br /><br />
         """
-        if decorations:
-            result = result + """
-        <div align="right"><font size="-1">
-         Problems?  Frustrations? <a href="/problems.html">Click here.</a>
-        </font></div>
-        """
+
         return result + self.GenerateStartScript(self.common_javascript) \
                + self.GenerateEndScript() + "</body>"
 
@@ -332,18 +320,6 @@ class DtmlPage:
         return make_button_for_request(title, request, css_class)
 
 
-    def MakeMenuItem(self, description, url):
-        """Generate HTML for a menu item.
-
-        'description' -- The name of the menu item.
-        
-        'url' -- The URL to which the user will be sent if this item
-        is selected."""
-        
-        return ('''<option value="%s">%s</option>'''
-                % (url, description))
-    
-        
     def MakeImageUrl(self, image):
         """Generate a URL for an image."""
 
@@ -456,12 +432,6 @@ class WebRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         # Store the client's IP address with the request.
         request.client_address = self.client_address[0]
         self.__HandleRequest(request)
-        self.wfile.flush()
-        try:
-            self.connection.shutdown(1)
-        except:
-            # Probably a network error.
-            pass
 
 
     def do_POST(self):
@@ -492,22 +462,10 @@ class WebRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             # Store the client's IP address with the request.
             request.client_address = self.client_address[0]
             self.__HandleRequest(request)
-        elif content_type == "text/xml":
-            # Check if this request corresponds to an XML-RPC invocation.
-            if self.server.IsXmlRpcUrl(self.path):
-                self.__HandleXmlRpcRequest()
-            else:
-                self.send_response(400, "Unexpected request (not XML/RPC).")
         else:
             self.send_response(400,
                                "Unexpected request (POST of %s)."
                                % content_type)
-        try:
-            self.wfile.flush()
-            self.connection.shutdown(1)
-        except:
-            # Probably a network error.
-            pass
 
 
     def __HandleScriptRequest(self, request):
@@ -529,7 +487,7 @@ class WebRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             return
         except SystemExit:
             self.server.RequestShutdown()
-            script_output = ("<html><b>Thank you for using %s.</b></html>"
+            script_output = ("<html><b>%s shutdown.</b></html>"
                              % qm.common.program_name)
         except:
             # Oops, the script raised an exception.  Show
@@ -566,42 +524,6 @@ class WebRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             pass
         
 
-    def __HandleXmlRpcRequest(self):
-        content_length = int(self.headers["Content-length"])
-        content = self.rfile.read(content_length)
-        arguments, method_name = xmlrpclib.loads(content)
-        # Do we have a matching method?
-        try:
-            method = self.server.GetXmlRpcMethod(method_name)
-        except KeyError:
-            # No matching method; report the error.
-            response = xmlrpclib.Fault(2, "no method: %s" % method_name)
-        else:
-            try:
-                # Invoke the method.
-                response = apply(method, arguments, {})
-                if not isinstance(response, types.TupleType):
-                    response = (response, )
-            except:
-                # The method raised an exception.  Send an XML-RPC
-                # fault response. 
-                exc_info = sys.exc_info()
-                response = xmlrpclib.Fault(1, "%s : %s"
-                                           % (exc_info[0], exc_info[1]))
-                # For debugging, dump exceptions out if verbose.
-                message = diagnostic.error(
-                    "xmlrpc exception", exception=format_exception(exc_info))
-                common.print_message(3, message)
-
-        # Encode the reponse.
-        response_string = xmlrpclib.dumps(response)
-        # Send it off.
-        self.send_response(200)
-        self.send_header("Content-type", "text/xml")
-        self.end_headers()
-        self.wfile.write(response_string)
-
-
     def __HandleFileRequest(self, request, path):
         # There should be no query arguments to a request for an
         # ordinary file.
@@ -610,7 +532,7 @@ class WebRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             return
         # Open the file.
         try:
-            file = open(path, "r")
+            file = open(path, "rb")
         except IOError:
             # Send a generic 404 if there's a problem opening the file.
             self.send_error(404, "File not found.")
@@ -627,7 +549,7 @@ class WebRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         """Process a retrieval request from the global page cache."""
 
         # Get the page from the cache.
-        page = get_from_cache(request, session_id=None)
+        page = self.server.GetCachedPage(request)
         # Send it.
         self.send_response(200)
         self.send_header("Content-Type", "text/html")
@@ -648,7 +570,7 @@ class WebRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
             self.send_error(400, "Missing session ID.")
             return
         # Get the page from the cache.
-        page = get_from_cache(request, session_id)
+        page = self.server.GetCachedPage(request, session_id)
         # Send it.
         self.send_response(200)
         self.send_header("Content-Type", "text/html")
@@ -751,7 +673,7 @@ class HTTPServer(BaseHTTPServer.HTTPServer):
 
 
 class WebServer(HTTPServer):
-    """A web server that serves ordinary files, dynamic content, and XML-RPC.
+    """A web server that serves ordinary files and dynamic content.
 
     To configure the server to serve ordinary files, register the
     directories containing those files with
@@ -763,10 +685,6 @@ class WebServer(HTTPServer):
     URLs with 'RegisterScript'.  A request matching the URL exactly
     will cause the server to invoke the provided function.
 
-    To configure the server to service XML-RPC requests, pass an
-    'xml_rpc_path' to the '__init__' function and register methods
-    with 'RegisterXmlRpcMethod'.
-
     The web server resolves request URLs in a two-step process.
 
     1. The server checks if the URL matches exactly a script URL.  If
@@ -790,17 +708,13 @@ class WebServer(HTTPServer):
         information that would be difficult to reconstruct later.
 
     Pages may be shared across sessions, or may be specific to a
-    particular session.
-
-    The page cache is shared among all 'WebServer' instances.  Use the
-    'cache_page' function to insert a page into the cache."""
+    particular session."""
 
 
     def __init__(self,
                  port,
                  address="",
-                 log_file=sys.stderr,
-                 xml_rpc_path=None):
+                 log_file=sys.stderr):
         """Create a new web server.
 
         'port' -- The port on which to accept connections.  If 'port'
@@ -812,9 +726,6 @@ class WebServer(HTTPServer):
         'log_file' -- A file object to which to write log messages.
         If it's 'None', no logging.
 
-        'xml_rpc_path' -- The script path at which to accept XML-RPC
-        requests.   If 'None', XML-RPC is disabled for this server.
-
         The server is not started until the 'Bind' and 'Run' methods are
         invoked."""
 
@@ -823,8 +734,6 @@ class WebServer(HTTPServer):
         self.__log_file = log_file
         self.__scripts = {}
         self.__translations = {}
-        self.__xml_rpc_methods = {}
-        self.__xml_rpc_path = xml_rpc_path
         self.__shutdown_requested = 0
 
         self.RegisterScript("/problems.html", self._HandleProblems)
@@ -834,32 +743,21 @@ class WebServer(HTTPServer):
         self.RegisterPathTranslation(
             "/common.js", qm.get_share_directory("web", "common.js"))
 
+        self.__cache_dir = temporary_directory.TemporaryDirectory()
+        self.__cache_path = self.__cache_dir.GetPath()
+        os.mkdir(os.path.join(self.__cache_path, "sessions"), 0700)
+
+        # Create a temporary attachment store to process attachment data
+        # uploads.
+        self.__temporary_store = qm.attachment.TemporaryAttachmentStore()
+        self.RegisterScript(qm.fields.AttachmentField.upload_url,
+                            self.__temporary_store.HandleUploadRequest)
+
         # Don't call the base class __init__ here, since we don't want
         # to create the web server just yet.  Instead, we'll call it
         # when it's time to run the server.
 
 
-    def GetXmlRpcUrl(self):
-        """Return the URL at which this server accepts XML-RPC.
-
-        preconditions -- The server must be bound.
-
-        returns -- The URL, or 'None' if XML-RPC is disabled."""
-
-        if self.__xml_rpc_path is None:
-            return None
-        else:
-            # Use the explicitly-specified host name, if provided.
-            # Otherwise, determine it from the socket.
-            host_name = self.__address
-            if host_name == "":
-                host_name = self.server_name
-            # Get the port from the socket; that works fine.
-            port = self.server_port
-            # Construct the URL.
-            return "http://%s:%d%s" % (host_name, port, self.__xml_rpc_path) 
-
-
     def RegisterScript(self, script_path, script):
         """Register a dynamic URL.
 
@@ -901,46 +799,18 @@ class WebServer(HTTPServer):
 
           web_server.RegisterPathTranslation('/images', '/path/to/pictures')
 
-        the the URL 'http://my.server.com/images/big/tree.gif' will be
+        the URL 'http://my.server.com/images/big/tree.gif' will be
         mapped to the file path '/path/to/pictures/big/tree.gif'."""
 
         self.__translations[url_path] = file_path
 
 
-    def RegisterXmlRpcMethod(self, method, method_name=None):
-        """Register an XML-RPC method.
-
-        precoditions -- XML-RPC must be enabled.
-
-        'method' -- The method to call in response to the request.  It
-        must be callable.  The request arguments are passed to this
-        method's positional parameters.
-
-        'method_name' -- The RPC name to associate with this method.
-        If 'None', the name of 'method' is used."""
-
-        assert self.__xml_rpc_path is not None
-        assert callable(method)
-        # Take the name of 'method' if no name is provided.
-        if method_name is None:
-            method_name = method.__name__
-        # Map it.
-        self.__xml_rpc_methods[method_name] = method
-
-
     def IsScript(self, request):
         """Return a true value if 'request' corresponds to a script."""
 
         return self.__scripts.has_key(request.GetUrl())
 
 
-    def IsXmlRpcUrl(self, url):
-        """Return true if 'url' corresponds to an XML-RPC invocation."""
-
-        return self.__xml_rpc_path is not None \
-               and url == self.__xml_rpc_path
-
-
     def ProcessScript(self, request):
         """Process 'request' as a script.
 
@@ -977,17 +847,6 @@ class WebServer(HTTPServer):
         return None
 
 
-    def GetXmlRpcMethod(self, method_name):
-        """Return the method corresponding to XML-RPC 'method_name'.
-
-        returns -- A callable object corresponding to 'method_name'.
-
-        raises -- 'KeyError' if there is no mapping for
-        'method_name'."""
-
-        return self.__xml_rpc_methods[method_name]
-
-
     def Bind(self):
         """Bind the server to the specified address and port.
 
@@ -1022,8 +881,6 @@ class WebServer(HTTPServer):
             self.server_address = (self.__address, self.__port)
             self.server_bind()
             self.server_activate()
-            qm.common.print_message(2, "Web server active on %s:%d.\n"
-                                    % self.GetServerAddress())
         except socket.error, error:
             error_number, message = error
             if error_number == errno.EADDRINUSE:
@@ -1072,6 +929,190 @@ class WebServer(HTTPServer):
         return (self.server_name, self.server_port)
 
 
+    def GetTemporaryAttachmentStore(self):
+        """Return the 'AttachmentStore' used for new 'Attachment's.
+
+        returns -- The 'AttachmentStore' used for new 'Attachment's."""
+
+        return self.__temporary_store
+    
+        
+    def MakeButtonForCachedPopup(self,
+                                 label,
+                                 html_text,
+                                 request=None,
+                                 window_width=480,
+                                 window_height=240):
+        """Construct a button for displaying a cached popup page.
+
+        'label' -- The button label.
+
+        'html_text' -- The HTML source for the popup page.
+
+        'window_width' -- The width, in pixels, of the popup window.
+
+        'window_height' -- The height, in pixels, of the popup window.
+
+        returns -- HTML source for the button.  The button must be placed
+        within a form element."""
+
+        # Place the page in the page cache.
+        if request is None:
+            session_id = None
+        else:
+            session_id = request.GetSessionId()
+        page_url = self.CachePage(html_text, session_id).AsUrl()
+
+        return make_button_for_popup(label, page_url, window_width,
+                                     window_height)
+
+
+    def MakeConfirmationDialog(self, message, url):
+        """Generate JavaScript for a confirmation dialog box.
+
+        'url' -- The location in the main browser window is set to the URL
+        if the user confirms the action.
+
+        See 'make_popup_dialog_script' for a description of 'function_name'
+        and 'message' and information on how to use the return value."""
+
+        # If the user clicks the "Yes" button, advance the main browser
+        # page. 
+        open_script = "window.opener.document.location = %s;" \
+                      % make_javascript_string(url)
+        # Two buttons: "Yes" and "No".  "No" doesn't do anything.
+        buttons = [
+            ( "Yes", open_script ),
+            ( "No", None ),
+            ]
+        return self.MakePopupDialog(message, buttons, title="Confirm")
+
+
+    def MakePopupDialog(self, message, buttons, title=""):
+        """Generate JavaScript to show a popup dialog box.
+
+        The popup dialog box displays a message and one or more buttons.
+        Each button can have a JavaScript statement (or statements)
+        associated with it; if the button is clicked, the statement is
+        invoked.  After any button is clicked, the popup window is closed as
+        well.
+
+        'message' -- HTML source of the message to display in the popup
+        window.
+
+        'buttons' -- A sequence of button specifications.  Each is a pair
+        '(caption, script)'.  'caption' is the button caption.  'script' is
+        the JavaScript statement to invoke when the button is clicked, or
+        'None'.
+
+        'title' -- The popup window title.
+
+        returns -- JavaScript statements to show the dialog box, suiteable
+        for use as an event handler."""
+
+        # Construct the popup page.
+        page = make_popup_page(message, buttons, title)
+        page_url = self.CachePage(page).AsUrl()
+        # Construct the JavaScript variable and function.
+        return "window.open('%s', 'popup', 'width=480,height=200,resizable')" \
+               % page_url
+
+
+    def CachePage(self, page_text, session_id=None):
+        """Cache an HTML page.
+        
+        'page_text' -- The text of the page.
+        
+        'session_id' -- The session ID for this page, or 'None'.
+        
+        returns -- A 'WebRequest' object with which the cached page can be
+        retrieved later.
+        
+        If 'session_id' is 'None', the page is placed in the global page
+        cache.  Otherwise, it is placed in the session page cache for that
+        session."""
+
+        if session_id is None:
+            # No path was specified.  Place the file in the top directory.
+            dir_path = self.__cache_path
+            script_name = _page_cache_name
+        else:
+            # A session was specified.  Put the file in a subdirectory named
+            # after the session.
+            dir_path = os.path.join(self.__cache_path, "sessions", session_id)
+            script_name = _session_cache_name
+            # Create that directory if it doesn't exist.
+            if not os.path.isdir(dir_path):
+                os.mkdir(dir_path, 0700)
+
+        # Generate a name for the page.
+        global _counter
+        page_name = str(_counter)
+        _counter = _counter + 1
+        # Write it.
+        page_file_name = os.path.join(dir_path, page_name)
+        page_file = open(page_file_name, "w", 0600)
+        page_file.write(page_text)
+        page_file.close()
+
+        # Return a request for this page.
+        request = WebRequest(script_name, page=page_name)
+        if session_id is not None:
+            request.SetSessionId(session_id)
+        return request
+
+
+    def GetCachedPage(self, request, session_id=None):
+        """Retrieve a page from the page cache.
+
+        'request' -- The URL requesting the page from the cache.
+
+        'session_id' -- The session ID for the request, or 'None'.
+
+        returns -- The cached page, or a placeholder page if the page was
+        not found in the cache.
+
+        If 'session_id' is 'None', the page is retrieved from the global
+        page cache.  Otherwise, it is retrieved from the session page cache
+        for that session."""
+
+        page_file_name = self.__GetPathForCachedPage(request, session_id)
+        if os.path.isfile(page_file_name):
+            # Return the page.
+            return open(page_file_name, "r").read()
+        else:
+            # Oops, no such page.  Generate a placeholder.
+            return """
+            <html>
+             <body>
+              <h3>Cache Error</h3>
+              <p>You have requested a page that no longer is in the server's
+              cache.  The server may have been restarted, or the page may
+              have expired.  Please start over.</p>
+              <!-- %s -->
+             </body>
+            </html>
+            """ % url
+
+
+    def __GetPathForCachedPage(self, request, session_id):
+        """Return the path for a cached page.
+
+        'request' -- The URL requesting the page from the cache.
+
+        'session_id' -- The session ID for the request, or 'None'."""
+
+        if session_id is None:
+            dir_path = self.__cache_path
+        else:
+            # Construct the path to the directory containing pages in the
+            # cache for 'session_id'.
+            dir_path = os.path.join(self.__cache_path, "sessions", session_id)
+        # Construct the path to the file containing the page.
+        page_name = request["page"]
+        return os.path.join(dir_path, page_name)
+
+
     def HandleNoSessionError(self, request, message):
         """Handler when session is absent."""
 
@@ -2099,7 +2140,7 @@ def make_help_link(help_text_tag, label="Help", **substitutions):
     'substitutions' -- Substitutions to the help diagnostic."""
     
     # Construct the help text.
-    help_text = apply(diagnostic.help_set.Generate,
+    help_text = apply(diagnostic.get_help_set().Generate,
                       (help_text_tag, "help", None),
                       substitutions)
     # Convert it to HTML.
@@ -2141,57 +2182,6 @@ def make_help_link_html(help_text, label="Help"):
            help_page.GenerateEndScript())
 
 
-def make_popup_dialog(message, buttons, title=""):
-    """Generate JavaScript to show a popup dialog box.
-
-    The popup dialog box displays a message and one or more buttons.
-    Each button can have a JavaScript statement (or statements)
-    associated with it; if the button is clicked, the statement is
-    invoked.  After any button is clicked, the popup window is closed as
-    well.
-
-    'message' -- HTML source of the message to display in the popup
-    window.
-
-    'buttons' -- A sequence of button specifications.  Each is a pair
-    '(caption, script)'.  'caption' is the button caption.  'script' is
-    the JavaScript statement to invoke when the button is clicked, or
-    'None'.
-
-    'title' -- The popup window title.
-
-    returns -- JavaScript statements to show the dialog box, suiteable
-    for use as an event handler."""
-
-    # Construct the popup page.
-    page = make_popup_page(message, buttons, title)
-    page_url = cache_page(page).AsUrl()
-    # Construct the JavaScript variable and function.
-    return "window.open('%s', 'popup', 'width=480,height=200,resizable')" \
-           % page_url
-
-
-def make_confirmation_dialog(message, url):
-    """Generate JavaScript for a confirmation dialog box.
-
-    'url' -- The location in the main browser window is set to the URL
-    if the user confirms the action.
-
-    See 'make_popup_dialog_script' for a description of 'function_name'
-    and 'message' and information on how to use the return value."""
-
-    # If the user clicks the "Yes" button, advance the main browser
-    # page. 
-    open_script = "window.opener.document.location = %s;" \
-                  % make_javascript_string(url)
-    # Two buttons: "Yes" and "No".  "No" doesn't do anything.
-    buttons = [
-        ( "Yes", open_script ),
-        ( "No", None ),
-        ]
-    return make_popup_dialog(message, buttons, title="Confirm")
-
-
 def make_popup_page(message, buttons, title=""):
     """Generate a popup dialog box page.
 
@@ -2239,76 +2229,6 @@ def make_popup_page(message, buttons, title=""):
     return page
 
 
-def make_select(field_name,
-                items,
-                default_value,
-                item_to_text=str,
-                item_to_value=str,
-                form_name="form",
-                update_external_field=0):
-    """Construct HTML controls for selecting an item from a list.
-
-    'field_name' -- The name of the form control which will contain the
-    currently-selected item.  This is a hidden control; the actual HTML
-    select control is given a different name.
-
-    'items' -- A sequence of items.  The items may be of any type.
-
-    'default_value' -- The item which should initially be selected.  The
-    value must appear in 'items'.
-
-    'item_to_text' -- A function that returns the text representation of
-    an item.
-
-    'item_to_value' -- A function that returns the value representation
-    of an item.
-
-    'form_name' -- The name of the form that will contain the returned
-    controls. 
-
-    'update_external_field' -- If true, update a field named
-    'field_name' when the selection changes.  If false, a hidden field
-    named 'field_name' is included, and updated when selection changes.
-
-    returns -- The HTML text for the controls.  
-
-    For each item, 'item_to_text' and 'item_to_value' are used to
-    construct the text and value attributes for the corresponding HTML
-    select option."""
-
-    assert default_value in items
-
-    select_name = "_select_" + field_name
-    # Add a hidden control with the specified field name.  This
-    # input will contain the value of the currently-selected
-    # UID.
-    if update_external_field:
-        result = ''
-    else:
-        result = '<input type="hidden" name="%s" value="%s"/>' \
-                 % (field_name, item_to_value(default_value))
-    # Write the handler to update the hidden control when the
-    # selection changes in the select control.
-    on_change = "update_from_select(document.%s.%s, " \
-                "document.%s.%s)" \
-                % (form_name, select_name, form_name, field_name)
-    # Generate a '<select>' control droplist.
-    result = result + '<select name="%s" onchange="%s">\n' \
-             % (select_name, on_change)
-    # Generate an '<option>' element for each item.
-    for item in items:
-        # Specify the 'select' attribute if this item represents the
-        # default value.
-        if item == default_value:
-            is_selected = "selected"
-        else:
-            is_selected = ""
-        result = result + '<option value="%s" %s>%s</option>\n' \
-                 % (item_to_value(item), is_selected, item_to_text(item))
-    result = result + '</select>\n'
-    return result
-    
-
 def make_choose_control(field_name,
                         included_label,
                         included_items,
@@ -2497,178 +2417,6 @@ def make_button_for_popup(label,
     """ % locals()
 
 
-def make_button_for_cached_popup(label,
-                                 html_text,
-                                 request=None,
-                                 window_width=480,
-                                 window_height=240):
-    """Construct a button for displaying a cached popup page.
-
-    'label' -- The button label.
-
-    'html_text' -- The HTML source for the popup page.
-
-    'window_width' -- The width, in pixels, of the popup window.
-
-    'window_height' -- The height, in pixels, of the popup window.
-
-    returns -- HTML source for the button.  The button must be placed
-    within a form element."""
-
-    # Place the page in the page cache.
-    if request is None:
-        session_id = None
-    else:
-        session_id = request.GetSessionId()
-    page_url = cache_page(html_text, session_id).AsUrl()
-
-    return make_button_for_popup(label, page_url, window_width,
-                                 window_height)
-
-        
-def cache_page(page_text, session_id=None):
-    """Cache an HTML page.
-
-    'page_text' -- The text of the page.
-
-    'session_id' -- The session ID for this page, or 'None'.
-
-    returns -- A 'WebRequest' object with which the cached page can be
-    retrieved later.
-
-    If 'session_id' is 'None', the page is placed in the global page
-    cache.  Otherwise, it is placed in the session page cache for that
-    session."""
-
-    global _page_cache_path
-    global _counter
-
-    if _page_cache_path is None:
-        # This is the first time we're inserting anything in the cache.
-        # Set it up.
-        _page_cache_path = common.make_temporary_directory()
-        os.mkdir(os.path.join(_page_cache_path, "sessions"), 0700)
-        # Clean up the cache at exit.
-        cleanup_function = lambda path=_page_cache_path: \
-                           common.rmdir_recursively(path)
-        common.add_exit_function(cleanup_function)
-
-    if session_id is None:
-        # No path was specified.  Place the file in the top directory.
-        dir_path = _page_cache_path
-        script_name = _page_cache_name
-    else:
-        # A session was specified.  Put the file in a subdirectory named
-        # after the session.
-        dir_path = os.path.join(_page_cache_path, "sessions", session_id)
-        script_name = _session_cache_name
-        # Create that directory if it doesn't exist.
-        if not os.path.isdir(dir_path):
-            os.mkdir(dir_path, 0700)
-
-    # Generate a name for the page.
-    page_name = str(_counter)
-    _counter = _counter + 1
-    # Write it.
-    page_file_name = os.path.join(dir_path, page_name)
-    page_file = open(page_file_name, "w", 0600)
-    page_file.write(page_text)
-    page_file.close()
-
-    # Return a request for this page.
-    request = WebRequest(script_name, page=page_name)
-    if session_id is not None:
-        request.SetSessionId(session_id)
-    return request
-
-
-def _get_path_for_cache_page(request, session_id):
-    """Return the path for a cached page.
-
-    'request' -- The URL requesting the page from the cache.
-
-    'session_id' -- The session ID for the request, or 'None'."""
-    
-    if session_id is None:
-        dir_path = _page_cache_path
-    else:
-        # Construct the path to the directory containing pages in the
-        # cache for 'session_id'.
-        dir_path = os.path.join(_page_cache_path, "sessions", session_id)
-    # Construct the path to the file containing the page.
-    page_name = request["page"]
-    return os.path.join(dir_path, page_name)
-
-
-def get_from_cache(request, session_id=None):
-    """Retrieve a page from the page cache.
-
-    'request' -- The URL requesting the page from the cache.
-
-    'session_id' -- The session ID for the request, or 'None'.
-
-    returns -- The cached page, or a placeholder page if the page was
-    not found in the cache.
-
-    If 'session_id' is 'None', the page is retrieved from the global
-    page cache.  Otherwise, it is retrieved from the session page cache
-    for that session."""
-    
-    page_file_name = _get_path_for_cache_page(request, session_id)
-    if os.path.isfile(page_file_name):
-        # Return the page.
-        return open(page_file_name, "r").read()
-    else:
-        # Oops, no such page.  Generate a placeholder.
-        return """
-        <html>
-         <body>
-          <h3>Cache Error</h3>
-          <p>You have requested a page that no longer is in the server's
-          cache.  The server may have been restarted, or the page may
-          have expired.  Please start over.</p>
-          <!-- %s -->
-         </body>
-        </html>
-        """ % url
-
-
-def format_user_id(user_id):
-    """Generate HTML for a user ID."""
-
-    try:
-        # Look up the user in the user database.
-        user_item = user.database[user_id]
-    except KeyError:
-        # The user isn't in the database.  Oops.
-        return '<font color="red">%s</font>' % user_id
-    else:
-        # Have we already generated and cached a user information page
-        # for this user?
-        if not hasattr(user_item, "__page_cache_request"):
-            # No.  Generate it now.
-            user_page = DtmlPage.default_class(
-                "user.dtml",
-                user_id=user_id,
-                user=user_item)()
-            # Put it in the page cache.
-            user_page_request = cache_page(user_page)
-            # Attach it to the user item.
-            user_item.__page_cache_request = user_page_request
-        else:
-            # Yes, we generated the page previously. 
-            user_page_request = user_item.__page_cache_request
-
-        # Return a link that pops up a window display the user
-        # information page.
-        return '''
-        <a href="javascript: void(0);"
-           onclick="window.open('%s', 'popup',
-                                'height=200,width=300,resizable');"
-           class="userid">%s</a>''' \
-            % (user_page_request.AsUrl(), user_id)
-
-
 def format_color(red, green, blue):
     """Format an RGB color value for HTML.
 
@@ -2732,9 +2480,6 @@ sessions = {}
 _counter = 0
 """A counter for generating somewhat-unique names."""
 
-_page_cache_path = None
-"""The path to the page cache."""
-
 _page_cache_name = "page-cache"
 """The URL prefix for the global page cache."""
 
